require_relative "../spec_helper"


describe By2::Client do
  context("finding records by src or dst ip") do
    it "finds entries that match src ip" do
      records = By2::Client.new(["-i", "0.0.0.1"]).find_records

      records.count.should eql(1)
      records.first.ip_src.should eql("0.0.0.1")
    end

    it "finds entries that match src ip and src port" do
      records = By2::Client.new(["-i", "0.0.0.1:80"]).find_records

      records.count.should eql(1)
      records.first.ip_src.should eql("0.0.0.1")
      records.first.sport.should eql(80)
    end

    it "finds entries that match dst ip" do
      records = By2::Client.new(["-i", "0.0.0.4"]).find_records

      records.count.should eql(1)
      records.first.ip_dst.should eql("0.0.0.4")
    end

    it "finds entries that match dst ip and dst port" do
      records = By2::Client.new(["-i", "0.0.0.4:85"]).find_records

      records.count.should eql(1)
      records.first.ip_dst.should eql("0.0.0.4")
      records.first.dport.should eql(85)
    end

    it "finds entries that match dst or src ip" do
      records = By2::Client.new(["-i", "0.0.0.2"]).find_records

      records.count.should eql(2)
      records.all? do |r|
        ((r.ip_dst.eql?("0.0.0.2")) || (r.ip_src.eql?("0.0.0.2"))).should be_true
      end
    end

    it "finds entries that match dst ip and port or src ip and port" do
      records = By2::Client.new(["-i", "0.0.0.2:82"]).find_records
      records.count.should eql(1)
      records.first.sport.should eql(82)

      records = By2::Client.new(["-i", "0.0.0.2:81"]).find_records
      records.count.should eql(1)
      records.first.dport.should eql(81)
    end
  end

  context("finding records by src and dst ip") do
    it "finds entries that match src ip" do
      records = By2::Client.new(["-s", "0.0.0.1"]).find_records

      records.count.should eql(1)
      records.first.ip_src.should eql("0.0.0.1")
    end

    it "finds entries that match src ip and src port" do
      records = By2::Client.new(["-s", "0.0.0.1:80"]).find_records

      records.count.should eql(1)
      records.first.ip_src.should eql("0.0.0.1")
      records.first.sport.should eql(80)
    end

    it "finds entries that match dst ip" do
      records = By2::Client.new(["-d", "0.0.0.2"]).find_records

      records.count.should eql(1)
      records.first.ip_dst.should eql("0.0.0.2")
    end

    it "finds entries that match dst ip and dst port" do
      records = By2::Client.new(["-d", "0.0.0.2:81"]).find_records

      records.count.should eql(1)
      records.first.ip_dst.should eql("0.0.0.2")
      records.first.dport.should eql(81)
    end

    it "finds entries that match src ip and dst ip" do
      records = By2::Client.new(["-s", "0.0.0.1", "-d", "0.0.0.2"]).find_records

      records.count.should eql(1)
      records.first.ip_src.should eql("0.0.0.1")
      records.first.ip_dst.should eql("0.0.0.2")
    end

    it "finds entries that match src ip/port and dst ip/port" do
      records = By2::Client.new(["-s", "0.0.0.1:80", "-d", "0.0.0.2:81"]).find_records

      records.count.should eql(1)
      records.first.ip_src.should eql("0.0.0.1")
      records.first.ip_dst.should eql("0.0.0.2")
      records.first.sport.should eql(80)
      records.first.dport.should eql(81)
    end
  end

  context("by2 -n SRC_IP:SRC_PORT -> DST_IP:DST_PORT") do
    it "finds entries that match src ip/port and dst ip/port" do
      records = By2::Client.new(["-m", "0.0.0.1:80 -> 0.0.0.2:81"]).find_records

      records.count.should eql(1)
      records.first.ip_src.should eql("0.0.0.1")
      records.first.ip_dst.should eql("0.0.0.2")
      records.first.sport.should eql(80)
      records.first.dport.should eql(81)
    end
  end

  context("finding records by date") do
    it "finds on specific day" do
      today = By2::Utils.fdate(Date.today)

      records = By2::Client.new(["-t", "#{today}"]).find_records
      records.count.should eql(1)
    end

    it "finds on and after specific date" do
      four_days_ago = By2::Utils.fdate(4.days.ago)

      records = By2::Client.new(["-t", "#{four_days_ago}:"]).find_records
      records.count.should eql(3)
    end

    it "finds within date range" do
      today = By2::Utils.fdate(Date.today)
      four_days_ago = By2::Utils.fdate(4.days.ago)

      records = By2::Client.new(["-t", "#{four_days_ago}:#{today}"]).find_records
      records.count.should eql(3)
    end
  end

  context("finding records with combined options") do
    it "finds src or dst ip within date range" do
      four_days_ago = By2::Utils.fdate(4.days.ago)

      records = By2::Client.new(["-i", "0.0.0.3", "-t", "#{four_days_ago}:"]).find_records
      records.count.should eql(2)
    end

    it "finds src or dst ip/port within date range" do
      four_days_ago = By2::Utils.fdate(4.days.ago)

      records = By2::Client.new(["-i", "0.0.0.3:82", "-t", "#{four_days_ago}:"]).find_records
      records.count.should eql(1)
    end
  end
end
